@@ -8,5 +8,7 @@ pub mod memory;
 mod packet;
 pub mod tngtls;
 
-pub use client::AtCaClient;
-pub use command::{Block, Digest, Signature};
+pub use client::{AtCaClient, Memory};
+pub use command::{Block, Digest, PublicKey, Signature};
+pub use packet::CRC16;
+pub use signature;
